@@ -33,6 +33,10 @@
 #
 #
 
+try:
+    import appindicator
+except ImportError:
+    appindicator = None
 
 import gtk
 import pkg_resources
@@ -63,7 +67,11 @@ class SystemTray(component.Component):
             "separatormenuitem4"
         ]
         self.config.register_set_function("enable_system_tray",
-            self.on_enable_system_tray_set)
+                                          self.on_enable_system_tray_set)
+        # bit of a hack to prevent function from doing something on startup
+        self.__enabled_set_once = False
+        self.config.register_set_function("enable_appindicator",
+                                          self.on_enable_appindicator_set)
 
         self.max_download_speed = -1.0
         self.download_rate = 0.0
@@ -77,25 +85,10 @@ class SystemTray(component.Component):
 
     def enable(self):
         """Enables the system tray icon."""
-        log.debug("Enabling the system tray icon..")
         self.tray_glade = gtk.glade.XML(
             pkg_resources.resource_filename("deluge.ui.gtkui",
                                             "glade/tray_menu.glade"))
 
-        if deluge.common.windows_check() or deluge.common.osx_check():
-            self.tray = gtk.status_icon_new_from_pixbuf(
-                common.get_logo(32))
-        else:
-            try:
-                self.tray = gtk.status_icon_new_from_icon_name("deluge")
-            except:
-                log.warning("Update PyGTK to 2.10 or greater for SystemTray..")
-                return
-
-        self.tray.connect("activate", self.on_tray_clicked)
-        self.tray.connect("popup-menu", self.on_tray_popup)
-
-
         self.tray_glade.signal_autoconnect({
             "on_menuitem_show_deluge_activate": \
                 self.on_menuitem_show_deluge_activate,
@@ -112,20 +105,53 @@ class SystemTray(component.Component):
 
         self.tray_menu = self.tray_glade.get_widget("tray_menu")
 
-        self.tray_glade.get_widget("download-limit-image").set_from_file(
-            deluge.common.get_pixmap("downloading16.png"))
-        self.tray_glade.get_widget("upload-limit-image").set_from_file(
-            deluge.common.get_pixmap("seeding16.png"))
+        if appindicator and self.config["enable_appindicator"]:
+            log.debug("Enabling the Application Indicator..")
+            self.indicator = appindicator.Indicator (
+                "deluge", "deluge", appindicator.CATEGORY_APPLICATION_STATUS)
+            # Pass the menu to the Application Indicator
+            self.indicator.set_menu(self.tray_menu)
+
+            # Make sure the status of the Show Window MenuItem is correct
+            self._sig_win_hide = self.window.window.connect("hide", self._on_window_hide)
+            self._sig_win_show = self.window.window.connect("show", self._on_window_show)
+            if self.window.visible():
+                self.tray_glade.get_widget("menuitem_show_deluge").set_active(True)
+            else:
+                self.tray_glade.get_widget("menuitem_show_deluge").set_active(False)
+
+            # Show the Application Indicator
+            self.indicator.set_status(appindicator.STATUS_ACTIVE)
+
+        else:
+            log.debug("Enabling the system tray icon..")
+            if deluge.common.windows_check() or deluge.common.osx_check():
+                self.tray = gtk.status_icon_new_from_pixbuf(
+                    common.get_logo(32))
+            else:
+                try:
+                    self.tray = gtk.status_icon_new_from_icon_name("deluge")
+                except:
+                    log.warning("Update PyGTK to 2.10 or greater for SystemTray..")
+                    return
+
+            self.tray.connect("activate", self.on_tray_clicked)
+            self.tray.connect("popup-menu", self.on_tray_popup)
+
+            # For some reason these icons do not display in appindicator
+            self.tray_glade.get_widget("download-limit-image").set_from_file(
+                deluge.common.get_pixmap("downloading16.png"))
+            self.tray_glade.get_widget("upload-limit-image").set_from_file(
+                deluge.common.get_pixmap("seeding16.png"))
 
         client.register_event_handler("ConfigValueChangedEvent", self.config_value_changed)
-        if not client.connected():
-        # Hide menu widgets because we're not connected to a host.
-            for widget in self.hide_widget_list:
-                self.tray_glade.get_widget(widget).hide()
-
         if client.connected():
             # We're connected so we need to get some values from the core
             self.__start()
+        else:
+            # Hide menu widgets because we're not connected to a host.
+            for widget in self.hide_widget_list:
+                self.tray_glade.get_widget(widget).hide()
 
     def __start(self):
         if self.config["enable_system_tray"]:
@@ -136,6 +162,16 @@ class SystemTray(component.Component):
                 self.tray_glade.get_widget("menuitem_quitdaemon").hide()
                 self.tray_glade.get_widget("separatormenuitem4").hide()
 
+            # These do not work with appindicator currently and can crash Deluge.
+            # Related to Launchpad bug #608219
+            if appindicator and self.config["enable_appindicator"]:
+                self.hide_widget_list.remove("menuitem_download_limit")
+                self.hide_widget_list.remove("menuitem_upload_limit")
+                self.hide_widget_list.remove("separatormenuitem3")
+                self.tray_glade.get_widget("menuitem_download_limit").hide()
+                self.tray_glade.get_widget("menuitem_upload_limit").hide()
+                self.tray_glade.get_widget("separatormenuitem3").hide()
+
             # Show widgets in the hide list because we've connected to a host
             for widget in self.hide_widget_list:
                 self.tray_glade.get_widget(widget).show()
@@ -154,7 +190,7 @@ class SystemTray(component.Component):
         self.__start()
 
     def stop(self):
-        if self.config["enable_system_tray"]:
+        if self.config["enable_system_tray"] and not self.config["enable_appindicator"]:
             try:
                 # Hide widgets in hide list because we're not connected to a host
                 for widget in self.hide_widget_list:
@@ -162,15 +198,19 @@ class SystemTray(component.Component):
             except Exception, e:
                 log.debug("Unable to hide system tray menu widgets: %s", e)
 
-            self.tray.set_tooltip(_("Deluge\nNot Connected.."))
+            self.tray.set_tooltip(_("Deluge") + "\n" + _("Not Connected..."))
 
     def shutdown(self):
         if self.config["enable_system_tray"]:
-            self.tray.set_visible(False)
+            if appindicator and self.config["enable_appindicator"]:
+                self.indicator.set_status(appindicator.STATUS_PASSIVE)
+            else:
+                self.tray.set_visible(False)
 
     def send_status_request(self):
-        client.core.get_download_rate().addCallback(self._on_get_download_rate)
-        client.core.get_upload_rate().addCallback(self._on_get_upload_rate)
+        client.core.get_session_status([
+            "payload_upload_rate",
+            "payload_download_rate"]).addCallback(self._on_get_session_status)
 
     def config_value_changed(self, key, value):
         """This is called when we received a config_value_changed signal from
@@ -184,21 +224,23 @@ class SystemTray(component.Component):
             self.max_download_speed = max_download_speed
             self.build_tray_bwsetsubmenu()
 
-    def _on_get_download_rate(self, download_rate):
-        self.download_rate = deluge.common.fsize(download_rate)
-
     def _on_max_upload_speed(self, max_upload_speed):
         if self.max_upload_speed != max_upload_speed:
             self.max_upload_speed = max_upload_speed
             self.build_tray_bwsetsubmenu()
 
-    def _on_get_upload_rate(self, upload_rate):
-        self.upload_rate = deluge.common.fsize(upload_rate)
+    def _on_get_session_status(self, status):
+        self.download_rate = deluge.common.fsize(status["payload_download_rate"])
+        self.upload_rate = deluge.common.fsize(status["payload_upload_rate"])
 
     def update(self):
         if not self.config["enable_system_tray"]:
             return
 
+        # Tool tip text not available for appindicator
+        if appindicator and self.config["enable_appindicator"]:
+            return
+
         # Set the tool tip text
         max_download_speed = self.max_download_speed
         max_upload_speed = self.max_upload_speed
@@ -244,12 +286,29 @@ class SystemTray(component.Component):
         submenu_bwdownset.show_all()
         submenu_bwupset.show_all()
 
-    def disable(self):
-        """Disables the system tray icon."""
-        log.debug("Disabling the system tray icon..")
+        # Re-set the menu to partly work around Launchpad bug #608219
+        if appindicator and self.config["enable_appindicator"]:
+            self.indicator.set_menu(self.tray_menu)
+
+    def disable(self,invert_app_ind_conf=False):
+        """Disables the system tray icon or appindicator."""
         try:
-            self.tray.set_visible(False)
-            del self.tray
+            if invert_app_ind_conf:
+                app_ind_conf = not self.config["enable_appindicator"]
+            else:
+                app_ind_conf = self.config["enable_appindicator"]
+            if appindicator and app_ind_conf:
+                if hasattr(self, "_sig_win_hide"):
+                    self.window.window.disconnect(self._sig_win_hide)
+                    self.window.window.disconnect(self._sig_win_show)
+                    log.debug("Disabling the application indicator..")
+
+                self.indicator.set_status(appindicator.STATUS_PASSIVE)
+                del self.indicator
+            else:
+                log.debug("Disabling the system tray icon..")
+                self.tray.set_visible(False)
+                del self.tray
             del self.tray_glade
             del self.tray_menu
         except Exception, e:
@@ -270,11 +329,18 @@ class SystemTray(component.Component):
         else:
             self.disable()
 
+    def on_enable_appindicator_set(self, key, value):
+        """Called whenever the 'enable_appindicator' config key is modified"""
+        if self.__enabled_set_once:
+            self.disable(True)
+            self.enable()
+        self.__enabled_set_once = True
+
     def on_tray_clicked(self, icon):
         """Called when the tray icon is left clicked."""
         self.blink(False)
 
-        if self.window.active() or self.window.visible():
+        if self.window.active():
             self.window.hide()
         else:
             if self.config["lock_tray"]:
@@ -324,9 +390,6 @@ class SystemTray(component.Component):
         if self.config["lock_tray"] and not self.window.visible():
             self.unlock_tray()
 
-        if self.config["classic_mode"]:
-            client.daemon.shutdown()
-
         self.window.quit()
 
     def on_menuitem_quitdaemon_activate(self, menuitem):
@@ -334,13 +397,22 @@ class SystemTray(component.Component):
         if self.config["lock_tray"] and not self.window.visible():
             self.unlock_tray()
 
-        client.daemon.shutdown()
-        self.window.quit()
+        self.window.quit(shutdown=True)
 
     def tray_setbwdown(self, widget, data=None):
         self.setbwlimit(widget, _("Set Maximum Download Speed"), "max_download_speed",
             "tray_download_speed_list", self.max_download_speed, "downloading.svg")
 
+    def _on_window_hide(self, widget, data=None):
+        """_on_window_hide - update the menuitem's status"""
+        log.debug("_on_window_hide")
+        self.tray_glade.get_widget("menuitem_show_deluge").set_active(False)
+
+    def _on_window_show(self, widget, data=None):
+        """_on_window_show - update the menuitem's status"""
+        log.debug("_on_window_show")
+        self.tray_glade.get_widget("menuitem_show_deluge").set_active(True)
+
     def tray_setbwup(self, widget, data=None):
         self.setbwlimit(widget, _("Set Maximum Upload Speed"), "max_upload_speed",
             "tray_upload_speed_list", self.max_upload_speed, "seeding.svg")
@@ -395,13 +467,13 @@ class SystemTray(component.Component):
         vbox = gtk.VBox(spacing=5)
         hbox.pack_start(vbox, False)
 
-        label = gtk.Label(_("<b><big>Deluge is password protected!</big></b>"))
+        label = gtk.Label("<b><big>%s</big></b>" % _("Deluge is password protected!"))
         label.set_use_markup(True)
         label.set_alignment(0.0, 0.5)
         label.set_line_wrap(True)
         vbox.pack_start(label, False)
 
-        tlabel = gtk.Label(_("<i>Enter your password to continue</i>"))
+        tlabel = gtk.Label("<i>%s</i>" % _("Enter your password to continue"))
         tlabel.set_use_markup(True)
         tlabel.set_alignment(0.0, 0.5)
         tlabel.set_line_wrap(True)
